"""
Copyright 2021 A Job Thing Sdn Bhd.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
   http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Initial code: Ryan Febriansyah, 20-12-2021
"""
import json
from .client import Http


class Assert(Http):
    """
    Base class for collection assertion
    test. All of these listing method are implemented
    based on abstract method in Http class
    """

    def assert_is_ok(self, message: str):
        if self.response.status_code != 200:
            message = "The request didn't success"
            raise AssertionError(message)
        else:
            return print(message)

    def assert_is_failed(self, message: str):
        if self.response.status_code == 200:
            message = "The request should be failed"
            raise AssertionError(message)
        else:
            return print(message)

    def assert_is_headers(self, message: str):
        if not self.response.headers:
            message = "There's no headers in that request"
            raise AssertionError(message)
        else:
            return print(message)

    def assert_is_content_type(self, message: str):
        # this one is much more specific
        # to body information
        if not self.response.headers["Content-Type"]:
            message = "Perhaps 'content-type' wasn't set"
            raise AssertionError(message)
        else:
            return print(message)

    def assert_content_type_to_equal(self, value: str, message: str):
        # validate expected content-type
        # with actual result in response
        if value != self.response.headers["Content-Type"]:
            message = "The value of content-type doesn't match with the actual result"
            raise AssertionError(message)
        else:
            return print(message)

    def assert_is_2xx_status(self, message: str):
        if 200 <= self.response.status_code < 300:
            return print(message, f"The status code was : {self.response.status_code}")
        else:
            message = "The status got 2xx"
            raise AssertionError(message)

    def assert_is_3xx_status(self, message: str):
        if not 300 <= self.response.status_code < 400:
            message = "The status not 3xx"
            raise AssertionError(message)
        else:
            print(message)

    def assert_is_4xx_status(self, message: str):
        if not 400 <= self.response.status_code < 500:
            message = "The status not 4xx"
            raise AssertionError(message)
        else:
            print(message)

    def assert_is_5xx_status(self, message: str):
        if not 500 <= self.response.status_code < 600:
            message = "The status not 5xx"
            raise AssertionError(message)
        else:
            print(message)

    def assert_has_content(self, message: str):
        if self.response.content:
            print(message, f"The content was => {self.response.content}")
        else:
            message = "There's no content in the body"
            raise AssertionError(message)

    def assert_has_json(self, message: str):
        if self.response.json:
            print(message, f"The JSON body was => {self.response.json}")
        else:
            message = "The request has no JSON object"
            raise AssertionError(message)

    def assert_has_text(self, message: str):
        if self.response.text:
            print(message, f"The request has text => {self.response.text}")
        else:
            message = "The request has no text object"
            raise AssertionError(message)

    def assert_status_code_in(self, status_code: str, message: str):
        expected_result = [str(code) for code in status_code]
        actual_result = str(self.response.status_code)
        if actual_result in expected_result:
            return print(message)
        else:
            message = "The expected status code didn't match with actual result"
            raise AssertionError(message)

    def assert_status_code_not_in(self, status_code: str, message: str):
        expected_result = [str(code) for code in status_code]
        actual_result = str(self.response.status_code)
        if actual_result not in expected_result:
            return print(message)
        else:
            message = (
                "The expected status code (actually) did matched with actual result"
            )
            raise AssertionError(message)

    def assert_json_to_equal(self, obj, message: str):
        response_data = self.response.json()
        dumps = json.dumps(response_data, sort_keys=False)
        loads = json.loads(dumps)
        if loads == obj:
            return print(message)
        else:
            message = "There's no object that match"
            raise AssertionError(message)

    def assert_text_to_equal(self, obj, message: str):
        if self.response.text == obj:
            return print(message)
        else:
            message = "There's no (text) object that match"
            raise AssertionError(message)

    def assert_content_to_equal(self, obj, message: str):
        if self.response.content == obj:
            return print(message)
        else:
            message = "There's no content that match"
            raise AssertionError(message)

    def assert_response_time(self, duration: int, message: str):
        if self.response.elapsed.total_seconds() <= duration:
            return print(message)
        else:
            message = "The duration exceeds the limit"
            raise AssertionError(message)

    def assert_text_to_equal(self, obj: str, message: str):
        if self.response.text:
            return print(isinstance(obj, str)), message
        else:
            message = f"Str type doesn't match with {obj}"
            raise AssertionError(message)

    def assert_dict_to_equal(self, obj: dict, message: str):
        if self.response.json:
            return print(isinstance(obj, dict)), message
        else:
            message = f"Dict type doesn't match with {obj}"
            raise AssertionError(message)

    def assert_response_time_less(self, message: str):
        # this one actually inspired from postman
        # collection test script about getting response
        # whenever calling an API
        max_duration = 200
        if self.response.elapsed.total_seconds() <= max_duration:
            return print(message)
        else:
            message = "The duration exceeds the limit"
            raise AssertionError(message)

    def assert_expected_to_fail(self, message: str):
        if self.response.status_code in [200, 201]:
            return print(message)
        else:
            message = "Expected to be failed, but got success instead"
            raise AssertionError(message)

    # TODO: write assertion for validating JSON body



from base64 import b64encode
from typing import Optional
from requests.auth import AuthBase, HTTPDigestAuth, HTTPBasicAuth
from requests import models


class BearerAuth(AuthBase):
    """
    Custom bearer authentication
    that sub-classes from requests module
    """

    def __init__(self, token: str) -> None:
        self.token = token

        if not isinstance(token, str):
            raise TypeError("Bearer token must be string of object")

    def __call__(self, r: models.PreparedRequest) -> models.PreparedRequest:
        r.headers["Authorization"] = f"Bearer {self.token}"
        return r


class BasicAuthToken(AuthBase):
    """
    Custom basic auth with token,
    its differ with requests BasicAuth
    """

    def __init__(self, token: str) -> None:
        self.token = token

        if not isinstance(token, str):
            raise TypeError("Basic auth token must be string of object")

    def __call__(self, r: models.PreparedRequest) -> models.PreparedRequest:
        encode = b64encode(f"token: {self.token}".encode("utf-8")).decode("utf-8")
        r.headers["Authorization"] = f"Basic {encode}"
        return r


class ApiKeyAuth(AuthBase):
    """
    Custom API key auth that will append
    into headers, this one is influenced
    by Postman authorization
    """

    def __init__(
        self, key: Optional[str], value: Optional[str], add_to: str = "headers"
    ) -> None:
        self.key = key
        self.value = value
        self.add_to = add_to

        if not isinstance(add_to, str):
            raise TypeError("`add_to` parameter must be string object")

    def __eq__(self, other) -> bool:
        return all([
            self.key == getattr(other, "key"),
            self.value == getattr(other, "value")
        ])

    def __call__(self, r: models.Request) -> models.Request:
        if self.key and self.value is not None:
            if self.add_to == "headers":
                r.headers["Authorization"] = self.key, self.value
            elif self.add_to == "query_params":
                r.params = {f"{self.key} : {self.value}"}
            else:
                raise ValueError("There is no option to add API key")
        return r


class BasicAuth(HTTPBasicAuth):
    """
    this only inherit from requests module,
    and wrapped it into new method.
    """
    pass


class DigestAuth(HTTPDigestAuth):
    """
    this only inherit from requests module,
    and wrapped it into new method
    """
    pass



import unittest
from maritest.response import Response


class TestHttpResponse(unittest.TestCase):
    def test_http_response_format(self):
        response = Response(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        response.retriever(format="json")
        response.retriever(format="multipart")
        response.retriever(format="text")

        # no redirection, just checking the other output
        response.history_response()

    def test_http_history_redirects(self):
        response = Response(
            method="GET",
            url="http://github.com/",
            headers={"some_key": "some_value"},
            allow_redirects=True,
            logger=False,
        )
        response.history_response()


if __name__ == "__main__":
    unittest.main()



import unittest
from unittest.case import expectedFailure
from maritest.client import Http


class MockMethod(Http):
    # this is only way that testing
    # for method that supposed
    # to be implemented already
    def assert_is_ok(self):
        raise NotImplementedError

    def concrete_method(self):
        return True


class TestHttpClient(unittest.TestCase):
    def test_get_method(self):
        request = Http(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        self.assertEqual("GET", request.method)
        self.assertEqual(200, request.response.status_code)

    def test_post_method(self):
        request_body = {
            "userId": 1,
            "id": 1,
            "title": "sunt aut facere repellat provident occaecati excepturi optio reprehenderit",
            "body": "quia et suscipit\nsuscipit recusandae consequuntur expedita et cum\nreprehenderit molestiae ut ut quas totam\nnostrum rerum est autem sunt rem eveniet architecto",
        }
        request = Http(
            method="POST",
            url="https://jsonplaceholder.typicode.com/posts",
            headers={"some_key": "some_value"},
            proxies={"https": "https://google.com"},
            json=request_body,
            logger=False,
            timeout=True,
        )
        self.assertEqual("POST", request.method)
        self.assertEqual(201, request.response.status_code)
        self.assertEqual(None, request.proxy)

    def test_put_method(self):
        request_body = {
            "userId": 1,
            "id": 1,
            "title": "sunt aut facere repellat provident occaecati excepturi optio reprehenderit",
            "body": "quia et suscipit\nsuscipit recusandae consequuntur expedita et cum\nreprehenderit molestiae ut ut quas totam\nnostrum rerum est autem sunt rem eveniet architecto",
        }
        request = Http(
            method="PUT",
            url="https://jsonplaceholder.typicode.com/posts/1",
            headers={"some_key": "some_value"},
            proxies=None,
            json=request_body,
            logger=False,
        )
        self.assertEqual("PUT", request.method)
        self.assertEqual(200, request.response.status_code)

    def test_patch_method(self):
        request_body = {
            "userId": 1,
            "id": 1,
            "title": "sunt aut facere repellat provident occaecati excepturi optio reprehenderit",
            "body": "quia et suscipit\nsuscipit recusandae consequuntur expedita et cum\nreprehenderit molestiae ut ut quas totam\nnostrum rerum est autem sunt rem eveniet architecto",
        }
        request = Http(
            method="PATCH",
            url="https://jsonplaceholder.typicode.com/posts/1",
            headers={"some_key": "some_value"},
            proxies=None,
            json=request_body,
            logger=False,
        )
        self.assertEqual("PATCH", request.method)
        self.assertEqual(200, request.response.status_code)

    def test_delete_method(self):
        request = Http(
            method="DELETE",
            url="https://jsonplaceholder.typicode.com/posts/1",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        self.assertEqual("DELETE", request.method)
        self.assertEqual(200, request.response.status_code)

    # negative test for validate
    # "OPTIONS" method is supported or not
    # expected to fail
    @expectedFailure
    def test_options_method(self):
        request = Http(
            method="OPTIONS",
            url="https://jsonplaceholder.typicode.com/posts/1",
            headers={"some_key": "some_value"},
            proxies={"http": "http://google.com"},  # failed proxy
            logger=False,
        )

        self.assertRaises(NotImplementedError, request.method)

    def test_http_attribute(self):
        request = Http(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        self.assertIn("", request.__str__())  # this one only checking for string type?
        self.assertIn("", request.__repr__())
        self.assertEqual("https://jsonplaceholder.typicode.com/posts", request.url)
        self.assertEqual("GET", request.method)
        self.assertEqual({"some_key": "some_value"}, request.headers)
        self.assertFalse(request.suppress_warning)  # validate this param set to True
        self.assertTrue(request.logger)

    def test_mock_http_method(self):
        mock_method = MockMethod(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        self.assertTrue(mock_method.concrete_method())

    def test_abstract_http_method(self):
        # Http.__abstractmethods__ = set()

        request = Http(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts",
            headers={"some_key": "some_value"},
            logger=False,
        )

        # intentionally to be failed because
        # we tested for abstractmethod in Http class
        # not the real assertion method in Assert class
        with self.assertRaises(NotImplementedError):
            request.assert_is_failed("failed")
            request.assert_is_ok("supposed to be ok, but failed")


if __name__ == "__main__":
    unittest.main()



from abc import ABC
from json.decoder import JSONDecodeError

from .client import Http


class Response(Http, ABC):
    """
    Formatted HTTP response and returned
    as either dict, text or binary
    """

    def retriever(self, format: str = "json"):
        """
        Collective method to retrieve HTTP response
        and returned as related argument ex: json,
        multipart and text

        :param format: Parameter for choose what kind
        of response that want to returned, by default
        set as JSON

        Returned formatted HTTP response
        """
        if not type(format):
            raise TypeError("Parameter format must be string object")

        if format.lower() not in ["multipart", "text", "json"]:
            raise NotImplementedError("There's no format that match with argument")

        try:
            if self.response.status_code == 200:
                if format.lower() == "json":
                    json_response = self.response.json()
                    response_body = {
                        "response_body": json_response,
                        "response_status_code": self.response.status_code,
                        "response_time": str(self.response.elapsed.total_seconds())
                        + " ms",
                        "response_headers": self.response.headers,
                        "is_json": True,
                    }
                elif format.lower() == "multipart":
                    part_response = self.response.content
                    response_body = part_response.decode()
                elif format.lower() == "text":
                    response_body = self.response.text, self.response.encoding
                else:
                    # need request body to debug when the
                    # error is occur during requested HTTP target
                    response_body = {
                        "request": {
                            "request_url": self.url,
                            "request_method": self.method,
                            "request_headers": self.headers,
                        },
                        "response": {
                            "response_body": None,
                            "response_status_code": self.response.status_code,
                            "response_time": str(self.response.elapsed.total_seconds())
                            + " ms",
                            "response_headers": self.response.headers,
                            "is_json": (False, type(self.response)),
                        },
                    }
                print(response_body)
            else:
                print((None, self.response.status_code))
        except JSONDecodeError as e:
            raise Exception(f"Unable to decode the HTTP response {e}")
        except Exception as e:
            raise Exception(f"Another exception was occurs {e}")

    def history_response(self):
        """
        Formatted string output for url redirection history
        """
        count = len(self.response.history)
        try:
            if count > 0:
                response_count = count
                response_body = [
                    print(f"URL redirects : {response.url}")
                    for response in self.response.history
                ]
            else:
                response_count = 0
                response_body = "Either there's no URL redirect that can be counted or don't enable the parameter " \
                                "`allow_redirects` settings "
        except Exception as e:
            raise Exception(f"Another exception was occurs {e}")

        print(f"Count history : {response_count}", response_body)



import unittest
from unittest.case import expectedFailure
from maritest.assertion import Assert


class TestHttpAssertion(unittest.TestCase):
    def test_assert_failed(self):
        # this test case supposed to be failed
        request = Assert(
            method="GET",
            url="https://jsonplaceholder.typicode.com/postas",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        request.assert_is_failed("should be failed")
        request.assert_is_4xx_status("404 status perhaps?")

    def test_assert_success(self):
        request = Assert(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        request.assert_is_2xx_status("Status was raised")
        request.assert_is_ok("it's okay")

    def test_assert_content_type(self):
        request = Assert(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        request.assert_content_type_to_equal(
            "application/json; charset=utf-8", "sukses"
        )
        request.assert_is_content_type(
            f"The content type is {request.response.headers} "
        )

    def test_assert_response(self):
        request = Assert(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        request.assert_has_content("yeah it has content")
        request.assert_has_json("yeah, it must be right?")
        request.assert_has_text("yeah it perhaps has text")
        request.assert_is_headers("HTTP headers it a must!")

    def test_assert_other_assertion(self):
        request = Assert(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        request.assert_response_time(200, "should be not exceed the limit")
        request.assert_status_code_in(
            [200, 201], "should be exactly match with the status code"
        )
        request.assert_status_code_not_in([404, 403], "should not be in status code")

    def test_assert_to_equal(self):
        request = Assert(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts/100",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        expected_body = {
            "userId": 10,
            "id": 100,
            "title": "at nam consequatur ea labore ea harum",
            "body": "cupiditate quo est a modi nesciunt soluta\nipsa voluptas error itaque dicta in\nautem qui minus magnam et distinctio eum\naccusamus ratione error aut",
        }
        expected_body_content = b'{\n  "userId": 10,\n  "id": 100,\n  "title": "at nam consequatur ea labore ea harum",\n  "body": "cupiditate quo est a modi nesciunt soluta\\nipsa voluptas error itaque dicta in\\nautem qui minus magnam et distinctio eum\\naccusamus ratione error aut"\n}'
        request.assert_json_to_equal(expected_body, "This one should be success")
        request.assert_content_to_equal(expected_body_content, "sukses")

    def test_assert_type_to_equal(self):
        request = Assert(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts/100",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        request.assert_dict_to_equal(
            {"some_key": "some_value"}, "this one is a dict object"
        )
        request.assert_text_to_equal(b"", "this one is text object")

    # TODO: write assertion test for 3xx and 5xx status code
    @expectedFailure
    def test_assert_other_status(self):
        request = Assert(
            method="GET",
            url="https://jsonplaceholder.typicode.com/postsa/100",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        request.assert_is_3xx_status("supposed to be fail, but not receive 3xx status")
        request.assert_is_5xx_status("supposed to be fail, but not receive 5xx status")

    def test_assert_3xx_status(self):
        request = Assert(
            method="GET",
            url="https://httpbin.org/status/301",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
            allow_redirects=False,
        )
        request.assert_is_3xx_status("This one must be 3xx status")
        request.allow_redirects == False

    def test_assert_5xx_status(self):
        request = Assert(
            method="GET",
            url="https://httpstat.us/500",
            headers={"some_key": "some_value"},
            proxies="https://httpstat.us/200",
            logger=False,
            allow_redirects=True,
        )
        request.assert_is_5xx_status("This one must be 5xx status")
        request.allow_redirects == True

    def assert_response_time_less(self):
        request = Assert(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts/100",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        request.assert_response_time_less("Shouldn't be exceed the limit")

    def assert_expect_to_failed(self):
        request = Assert(
            method="GET",
            url="https://jsonplaceholder.typicode.com/posts/100",
            headers={"some_key": "some_value"},
            proxies=None,
            logger=False,
        )
        request.assert_expected_to_fail("This one is not failed, but must be pass")


if __name__ == "__main__":
    unittest.main()



"""
Copyright 2021 A Job Thing Sdn Bhd.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
   http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Initial code: Ryan Febriansyah, 03-12-2021
"""
try:
    import requests
except ImportError as e:
    raise Exception(f"Unable to importerd `requests` package {e}")

import logging
import warnings
import urllib3
import urllib.parse
from .version import __version__
from abc import abstractmethod
from requests.sessions import CaseInsensitiveDict
from requests.packages.urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter
from contextlib import contextmanager

# For our purposes,
# its only supported 5 HTTP method
ALLOWED_METHODS = ["GET", "PUT", "POST", "DELETE", "PATCH"]

get_specific_logger = logging.getLogger("Maritest Logger")


# TODO: separate this function calls
@contextmanager
def disable_warnings():
    # thanks stack overflow see at: https://stackoverflow.com/questions/27981545/suppress-insecurerequestwarning
    # -unverified-https-request-is-being-made-in-pytho
    with warnings.catch_warnings():
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        yield None


def response_hook(response: requests.Response, *args, **kwargs):
    return response.raise_for_status(), *args, kwargs


# TODO: also split this function callable
def default_headers():
    return CaseInsensitiveDict(
        {
            "User-Agent": f"maritest, {__version__}",
            "Accept": "*/*",
            "Connection": "keep-alive",
        }
    )


class Http:
    """
    This is a base class for HTTP client
    """

    def __init__(
        self,
        method: str,
        url: str,
        headers: dict,
        allow_redirects: bool = None,
        logger: bool = True,
        event_hooks: bool = False,
        retry: bool = True,
        supress_warning: bool = None,
        proxy: dict = None,
        **kwargs,
    ) -> None:
        """
        Constructur for request HTTP target, with
        several parameter to construct it :

        :param method: HTTP method verb, string type
        :param url: base url for HTTP target, string type
        :param headers: HTTP content headers, by default always
        set to dict object
        :param allow_redirects: Enable redirection to other
        HTTP target if previous one wasn't respond it. set None
        :param logger: Logger stream handler with formatted
        output of message, by default set True
        :param event_hooks: Given valid response, by default
        set to False
        :param retry: Enable retry mechanism with default total
        attempts up-to 3, by default set to True
        :param suppress_warning: Verification of SSL certificate, if
        set False, will suppressed warning message
        :param proxy: HTTP proxies configuration, by default
        always set to None, and must be configured in HTTPS
        :param kwargs: given by keyword argument

        Returned as HTTP response
        """
        # missing attributes :
        # session, certs, file, json, data
        self.method = method
        self.url = url
        self.headers = headers

        assert isinstance(method, str), "method must be string object"
        assert isinstance(url, str), "url schema must be string object"
        assert isinstance(headers, dict), "headers must be dict object"

        if logger:
            self.logger = get_specific_logger
            self.logger.propagate = False
            self.logger.setLevel(logging.DEBUG)

            # handler the logging event
            # and write the standart output
            logger_output = logging.StreamHandler()
            logger_output.setLevel(logging.DEBUG)

            # format the standart output
            # with timestamp of event, class name and levels
            logger_formatter = logging.Formatter(
                fmt="%(asctime)s : %(name)s : %(funcName)s : %(message)s",
                datefmt="%d-%m-%Y %I:%M:%S",
            )

            logger_output.setFormatter(logger_formatter)
            self.logger.addHandler(logger_output)
        else:
            # if logger wasn't setup
            # only get the specific logger name
            self.logger = get_specific_logger

        self.timeout = None
        self.response = None
        self.json = None
        self.data = None
        self.params = None
        self.files = None
        self.proxy = proxy
        self.allow_redirects = allow_redirects
        self.stream = False
        self.cert = None
        self.suppress_warning = (
            supress_warning  # by default set to True due staging environment
        )
        self.verify = None
        self.session = None

        # re-write this message
        # the output still returned as
        # interpolation format
        message = f"[DEBUG] HTTP Request {self.headers}, {self.timeout}"

        if logger:
            # why the heck am i validate
            # the logger twice ??
            self.logger.info(f"[INFO] HTTP Request {self.method} | {self.url}")
            self.logger.debug(msg=message)

        if headers is None:
            # enforcing headers alwasy
            # wrap themselves with dict type
            # merge with pre-defined headers
            self.headers = default_headers()

        if self.timeout is None:
            self.timeout = 120

        if method is not None:
            self.method = method.upper()

        if self.method not in [index for index in ALLOWED_METHODS]:
            raise NotImplementedError(f"Currently {self.method} method not supported")

        if self.response is None:
            self.response = requests.Response

        if self.data is None:
            self.data = {}

        if self.params is None:
            self.params = {}

        # new attribute to call the request session instance
        if self.session is None:
            self.session = requests.Session()

        # by default, using proxies only
        # for HTTPS over HTTP connection
        if proxy is not None:
            if "https" not in [key for key in self.proxy.keys()]:
                raise ConnectionError(
                    "Proxy connection must be configured HTTPS over HTTP"
                )
            else:
                # elsewhere, update the new conf
                # for proxy before merge into request
                # TODO: differentiate the sessions attribute itself
                self.proxy = self.session.proxies.update(proxy)

        if supress_warning is not None:
            if supress_warning:
                warnings.warn(
                    "parameter `suppressed_warning` will be deprecated and no longer use in the next release "
                    "consider to add certification path instead or always enable the SSL verification issue "
                )
                self.suppress_warning = True
                self.logger.warning("[WARNING] SSL verification status is disabled")
            else:
                # move disable warnings in here
                disable_warnings()
                self.suppress_warning = False
                self.logger.info("[INFO] SSL verification status is enabled")

        # wrap it our request
        # and prepare it first before
        # send it immediately
        request = requests.Request(
            method=self.method,
            url=self.url,
            headers=self.headers,
            params=self.params,
            data=self.data,
            json=self.json,
            files=self.files,
        )
        prepare_request = request.prepare()

        # before final request, check
        # whether environment has proxies protocol or not
        # if yes, then merged it as one
        update_request = self.session.merge_environment_settings(
            url=self.url,
            proxies=self.proxy,
            stream=self.stream,
            verify=self.suppress_warning,
            cert=self.cert,
        )
        kwargs = {"allow_redirects": self.allow_redirects, "timeout": self.timeout}
        kwargs.update(update_request)

        try:
            with self.session as s:
                if retry:
                    self.retry = Retry(
                        total=3,
                        status_forcelist=[429, 500, 502, 503, 504],
                        method_whitelist=["GET", "POST"],
                        backoff_factor=0.3,
                    )
                    adapter = HTTPAdapter(max_retries=self.retry)
                    if urllib.parse.urlparse(self.url).scheme == "http":
                        s.mount("https://", adapter)
                    else:
                        # only given a log warning for user
                        s.mount("http://", adapter)
                        self.logger.warning(
                            f"[WARNING] you're going to mounted unverified (HTTP) protocol"
                        )
                else:
                    self.logger.info("[INFO] HTTP retry method might be turned it off")
                self.response = s.send(request=prepare_request, **kwargs)
        except requests.exceptions.Timeout as e:
            # temporary using requests exception
            # TODO: make base class for custom exceptio
            raise Exception(f"HTTP Request was timeout {e}")
        except requests.exceptions.ConnectionError as e:
            raise Exception(f"HTTP Request was failed {e}")
        except Exception as e:
            raise Exception(f"Other exception was occur {e}")
        finally:
            pass

        self.logger.info(f"[INFO] HTTP Response {self.response.status_code}")
        self.logger.debug(f"[DEBUG] HTTP Response Header {self.response.headers}")
        # self.logger.debug(f"[DEBUG] HTTP Response Content {self.response.content}")

        if event_hooks:
            # if event hooks was set to True
            # call the valid response name instead
            # only call the message from related assertion
            self.response.raise_for_status()
            s.hooks["response"] = [response_hook]

    def __str__(self) -> str:
        if self.method and self.url is not None:
            return f"You're using Maritest with HTTP Request {self.url} | {self.method}"

    def __repr__(self) -> str:
        return f"<Http:{self.method}=>{self.url}>"

    def __eq__(self, other) -> bool:
        return (
            isinstance(other, self.__class__)
            and self.method == other.method
            and self.url == other.url
            and self.headers == other.headers
            and self.timeout == other.timeout
            and self.response == other.response
        )

    def __enter__(self):
        return self

    def __exit__(self):
        # exit the connection pooling
        # after send the final request
        self.response.close()

    # add setter-getter only for
    # base_url or HTTP method
    @property
    def url(self) -> str:
        return self._url

    @url.setter
    def url(self, value: str) -> str:
        self._url = value

    @url.deleter
    def url(self) -> None:
        del self._url

    @property
    def method(self):
        return self._method

    @method.setter
    def method(self, value: str) -> str:
        self._method = value

    @method.deleter
    def method(self) -> None:
        del self._method

    # this one act as an
    # instances for assertion tests
    # that have similar function like in pytest, unittest
    # or in postman testscript. For example:
    # test_foo = Http().assert_is_ok()
    @abstractmethod
    def assert_is_ok(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_is_failed(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_is_headers(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_is_content_type(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_content_type_to_equal(self, value: str, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_is_2xx_status(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_is_3xx_status(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_is_4xx_status(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_is_5xx_status(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_has_content(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_has_json(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_has_text(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_status_code_in(self, status_code: str, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_status_code_not_in(self, status_code: str, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_json_to_equal(self, obj, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_text_to_equal(self, obj, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_content_to_equal(self, obj, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_response_time(self, duration: int, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_text_to_equal(self, obj: str, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_dict_to_equal(self, obj: dict, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_response_time_less(self, message: str):
        raise NotImplementedError

    @abstractmethod
    def assert_expected_to_fail(self, message: str):
        raise NotImplementedError



import io
import os
import re
from setuptools import setup, find_packages

with io.open("maritest/version.py", "rt", encoding="utf-8") as f:
    version = re.search(r"__version__ = '(.*?)'", f.read()).group(1)

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="maritest",
    version=version,
    author="Ryan Febriansyah",
    author_email="ryanfebriansyah72@gmail.com",
    description="API testing framework for simplify assertion",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    url="https://github.com/sodrooome/maritest",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    python_requires=">=3.7",
    install_requires=[
        "requests",
    ],
)
